module ActsAsPasting
  module Pastable
    extend ActiveSupport::Concern

    # 带 prefix 的 pasted_with，独立于不带 prefix 的之外
    included do
      has_many :pastings, as: :pasteable, class_name: 'ActsAsPasting::Pasting'
      has_many :hosts, through: :pastings

      # ary => [[klass, id], [klass, id]]
      # ary => [obj, obj]
      scope :pasted_with_any, ->(ary, prefix: '') {
        ary = parsed_condition_ary(ary)
        arel = Arel::Table.new(:acts_as_pasting_pastings, as: "acts_as_pasting_pastings_#{SecureRandom.hex(10)}")
        source_arel = arel_table
        join_condition = ary.group_by(&:first).map do |(klass, value)|
          ids = value.map(&:last)
          arel[:pasted_type].eq(klass.constantize.base_class.name).
            and(arel[:pasted_id].in(ids)).
            and(arel[:pasteable_id].eq(source_arel[:id])).
            and(arel[:type].eq(prefix))
        end.reduce(:or)
        joins(arel.join(arel).on(join_condition).join_sources).distinct
      }

      scope :pasted_with_all, ->(ary, prefix: '') {
        ary = parsed_condition_ary(ary)
        ary.reduce(where(nil)) do |association, value|
          association.pasted_with_any([value], prefix: prefix)
        end
      }
    end

    module ClassMethods
      def acts_as_pastable associations, prefix: '', **options
        pasting_name = prefix.blank? ? "#{associations}_pastings" : [associations, prefix, 'pastings'].join('_').to_sym
        has_many(pasting_name, -> { where(type: prefix) }, as: :pasteable, class_name: 'ActsAsPasting::Pasting')
        has_many(associations, through: pasting_name, source: :pasted, **options)
      end
    end
  end
end
